import json
from datetime import datetime, timezone
from typing import Any, Dict, cast
from typing_extensions import override

from masterpiece.mqtt import MqttMsg
from juham_core import Juham, MasterPieceThread, JuhamThread
from .shelly import Shelly


class ShellyPlusAddOnSimulatorThread(MasterPieceThread):
    """Thread simulating Shelly Plus 1 wifi relay with four temperature
    sensors."""

    def __init__(self, topic: str = "", interval: float = 60) -> None:
        """Construct thread for simulating data from Shelly1Plus sensors.

        Args:
            topic (str, optional): MQTT topic to post the sensor readings. Defaults to None.
            interval (float, optional): Interval specifying how often the sensor is read. Defaults to 60 seconds.
        """
        super().__init__(None)
        self.sensor_topic = topic
        self.interval: float = interval
        self.temp: float = 55

    @override
    def update_interval(self) -> float:
        return self.interval

    @override
    def update(self) -> bool:
        super().update()
        ts = datetime.now(timezone.utc).timestamp()
        self.temp = self.temp + 0.1
        if self.temp > 70:
            self.temp = 50

        data: dict[str, Any] = {
            "method": "NotifyStatus",
            "params": {
                "ts": ts,
                "temperature:100": {
                    "tC": self.temp,  # Temperature in Celsius for sensor1
                },
                "temperature:101": {
                    "tC": self.temp * 0.9,  # Temperature in Celsius for sensor2
                },
                "temperature:102": {
                    "tC": self.temp * 0.8,  # Temperature in Celsius for sensor2
                },
                "temperature:103": {
                    "tC": self.temp * 0.7,  # Temperature in Celsius for sensor2
                },
            },
        }
        msg = json.dumps(data)
        self.publish(self.sensor_topic, msg)
        return True


class ShellyPlusAddOnSimulator(JuhamThread, Shelly):
    """Simulator for ShellyPlus1 wifi relay.

    Spawns an asynchronous thread to generate data from temperature
    sensors.
    """

    workerThreadId: str = ShellyPlusAddOnSimulatorThread.get_class_id()
    shelly_topic: str = "/events/rpc"
    update_interval: float = 60

    def __init__(
        self,
        name: str = "shellyplus1-simulator",
        topic: str = "",
        interval: float = 60.0,
        mqtt_prefix: str = "shellyplus1-a0a3b3c309c4",
    ) -> None:
        """Create Shelly Plus 1 Simulator.

        Args:
            name (str, optional): name of the object. Defaults to 'rhomewizardwatermeter'.
            topic (str, optional): shelly device specific topic. Defaults to None.
            interval (float, optional): _description_. Defaults to None.
            shelly_mqtt_prefix (str, optional): MQTT topic prefix to which to publish shelly events
        """
        super().__init__(name)
        self.mqtt_prefix = mqtt_prefix
        self.active_liter_lpm = -1
        self.update_ts = None
        if topic:
            self.topic = topic
        self.interval = interval
        self.sensor_topic = self.mqtt_prefix + self.shelly_topic

    @override
    def on_connect(self, client: object, userdata: Any, flags: int, rc: int) -> None:
        super().on_connect(client, userdata, flags, rc)
        if rc == 0:
            self.subscribe(self.mqtt_prefix + self.shelly_topic)

    @override
    def on_message(self, client: object, userdata: Any, msg: MqttMsg) -> None:
        if msg.topic == self.sensor_topic:
            em = json.loads(msg.payload.decode())
            self.on_sensor(em)
        else:
            super().on_message(client, userdata, msg)

    def on_sensor(self, em: dict[str, Any]) -> None:
        """Handle data coming from the the Shelly sensors.

        Simply log the event to indicate the presense of simulated device.
        Args:
            em (dict): data from the sensor
        """
        pass

    @override
    def run(self) -> None:
        self.worker = cast(
            ShellyPlusAddOnSimulatorThread,
            Juham.instantiate(ShellyPlusAddOnSimulatorThread.get_class_id()),
        )
        self.worker.sensor_topic = self.sensor_topic
        self.worker.interval = self.update_interval
        super().run()

    @override
    def to_dict(self) -> Dict[str, Any]:
        data: Dict[str, Any] = super().to_dict()
        data["_shellyplus1simulator"] = {
            "shelly_topic": self.shelly_topic,
        }
        return data

    @override
    def from_dict(self, data: Dict[str, Any]) -> None:
        super().from_dict(data)
        if "_shellyplus1simulator" in data:
            for key, value in data["_shellyplus1simulator"].items():
                setattr(self, key, value)



import json
import time

from typing import Any, Dict, cast
from typing_extensions import override
from masterpiece.mqtt import MqttMsg
from juham_core import Juham, MasterPieceThread, JuhamThread

from .shelly import Shelly


class ShellyMotionSimulatorThread(MasterPieceThread):
    """Thread simulating Shelly Plus 1 wifi relay with four temperature
    sensors."""

    def __init__(self, topic: str = "", interval: float = 60) -> None:
        """Construct thread for simulating data from Shelly motion sensors.

        Args:
            topic (str, optional): MQTT topic to post the sensor readings. Defaults to None.
            interval (float, optional): Interval specifying how often the sensor is read. Defaults to 60 seconds.
        """
        super().__init__(None)
        self.shelly_topic: str = topic
        self.interval: float = interval

    @override
    def update_interval(self) -> float:
        return self.interval

    @override
    def update(self) -> bool:
        super().update()

        m: dict[str, Any] = {
            "tmp": {"value": 22.5},  # Room temperature value
            "sensor": {
                "vibration": True,  # Vibration status
                "motion": False,  # Motion status
            },
            "unixtime": int(time.time()),
        }

        msg = json.dumps(m)
        self.publish(self.shelly_topic, msg, 1, True)
        return True


class ShellyMotionSimulator(JuhamThread, Shelly):
    """Simulator for Shelly Motion 2 - a wifi motion sensor. Spawns a thread
    to generate MQTT messages as if they origin from the actual Shelly motion sensor"""

    workerThreadId = ShellyMotionSimulatorThread.get_class_id()
    shelly_topic = "shellies/shellymotion2/info"
    update_interval = 60

    def __init__(
        self,
        name: str = "shellymotionsensor",
        topic: str = "",
        interval: float = 60,
    ) -> None:
        """Create Shelly motion sensor simulator.

        Args:
            name (str, optional): Name of the object. Defaults to 'shellymotionsensor'.
            topic (str, optional): MQTT topic to publish motion sensor events. Defaults to None.
            interval (float, optional): interval between events, in seconds. Defaults to None.
        """
        super().__init__(name)
        self.active_liter_lpm = -1
        self.update_ts = None
        if topic:
            self.topic = topic
        if interval:
            self.interval = interval

    @override
    def on_connect(self, client: object, userdata: Any, flags: int, rc: int) -> None:
        super().on_connect(client, userdata, flags, rc)
        if rc == 0:
            self.subscribe(self.shelly_topic)

    @override
    def on_message(self, client: object, userdata: Any, msg: MqttMsg) -> None:
        if msg.topic == self.shelly_topic:
            em = json.loads(msg.payload.decode())
            self.on_sensor(em)
        else:
            super().on_message(client, userdata, msg)

    def on_sensor(self, em: dict[str, Any]) -> None:
        """Handle data coming from the Shelly motion sensors.

        Simply log the event to indicate the presense of simulated device.
        Args:
            em (dict): data from the sensor
        """
        self.debug(f"Motion sensor sensor {em}")

    @override
    def run(self) -> None:
        self.worker = cast(
            ShellyMotionSimulatorThread,
            Juham.instantiate(ShellyMotionSimulatorThread.get_class_id()),
        )
        self.worker.shelly_topic = self.shelly_topic
        self.worker.interval = self.update_interval
        super().run()

    @override
    def to_dict(self) -> Dict[str, Any]:
        data: Dict[str, Any] = super().to_dict()
        data["_shellymotionsimulator"] = {"shelly_topic": self.shelly_topic}
        return data

    @override
    def from_dict(self, data: Dict[str, Any]) -> None:
        super().from_dict(data)
        if "_shellymotionsimulator" in data:
            for key, value in data["_shellymotionsimulator"].items():
                setattr(self, key, value)



from typing_extensions import override
from masterpiece import Plugin, Composite
from .shellymotion_simulator import ShellyMotionSimulator
from .shellyplusaddon_simulator import ShellyPlusAddOnSimulator

# from .shelly1g3 import Shelly1G3
# from .shellypro3em import ShellyPro3EM
# from .shellymotion import ShellyMotion


class ShellyPlugin(Plugin):
    """Plugin class for installing and instantiating Shelly's into the host application."""

    enable_motion_simulator: bool = False  # motion sensor
    enable_plusaddon_simulator: bool = False  # temperature and humidity sensors

    def __init__(self, name: str = "openweather_map") -> None:
        """Create systemstatus object."""
        super().__init__(name)

    @override
    def install(self, app: Composite) -> None:
        if self.enable_motion_simulator:
            app.add(ShellyMotionSimulator())
        if self.enable_plusaddon_simulator:
            app.add(ShellyPlusAddOnSimulator())
